package kal.base.jdf.serv;

/**
 * @(#) HttpUtility.java
 * Copyright 1999-2000 by  LG-EDS Systems, Inc.,
 * Information Technology Group, Application Architecture Team, 
 * Application Intrastructure Part.
 * 236-1, Hyosung-2dong, Kyeyang-gu, Inchun, 407-042, KOREA.
 * All rights reserved.
 * 
 * NOTICE !      You can copy or redistribute this code freely, 
 * but you should not remove the information about the copyright notice 
 * and the author.
 * 
 * @author  WonYoung Lee, wyounglee@lgeds.lg.co.kr.
 */
import java.lang.reflect.*;
import java.util.Enumeration;
import java.util.Vector;
import javax.servlet.http.HttpServletRequest;

public class HttpUtility {
	/**
	 * You can't call the constructor.
	 */
	private HttpUtility() {}

	/**
	 * Decode a string from <code>x-www-form-urlencoded</code> format.
	 *
	 * @param   s   an encoded <code>String</code> to be translated.
	 * @return  the original <code>String</code>.
	 * @see		java.net.URLEncoder#encode(java.lang.String)
	 */
	public static String decode(String s) {
		return java.net.URLEncoder.encode(s);
	}

	/**
	 * Translates a string into <code>x-www-form-urlencoded</code> format.
	 *
	 * @param   s   <code>String</code> to be translated.
	 * @return  the translated <code>String</code>.
	 * @see		java.net.URLEncoder#encode(java.lang.String)
	 */
	public static String encode(String s) {
		return java.net.URLEncoder.encode(s);
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static Box getBox(HttpServletRequest req)  {
		Box box = new Box("requestbox");

		Enumeration e = req.getParameterNames();
		while(e.hasMoreElements()){
			String key = (String)e.nextElement();
			box.put(key, req.getParameterValues(key));
		}
		return box;
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static Box getBoxFromCookie(HttpServletRequest req)  {
		Box cookiebox = new Box("cookiebox");
		javax.servlet.http.Cookie[] cookies = req.getCookies();
		if ( cookies == null ) return cookiebox;

		for(int i=0; cookies != null && i< cookies.length; i++) {
			String key = cookies[i].getName();
			String value = cookies[i].getValue();
			if ( value == null ) value = "";
			String[] values = new String[1];
			values[0] = value;
			cookiebox.put(key,values);
		}
		return cookiebox;
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static SessionBox getNewSessionBox(HttpServletRequest req)  {
		return getNewSessionBox(req, "shared session box");
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static SessionBox getNewSessionBox(HttpServletRequest req, String name)  {
		
		javax.servlet.http.HttpSession  session = req.getSession(true);
		
		SessionBox sessionbox = null;
		
		if (! session.isNew()) {
			//Object o = session.getValue(name);
			Object o = session.getAttribute(name); // by chkim(2002-04-26)
			if ( o != null ) {
				//session.removeValue(name);
				session.removeAttribute(name); // by chkim(2002-04-26)
			}
		}

		if ( sessionbox == null ) {
			sessionbox = new SessionBox(session, name);
			//session.putValue(name, sessionbox);
			session.setAttribute(name, sessionbox); // by chkim(2002-04-26)
		}
		
		Enumeration e = req.getParameterNames();
		while(e.hasMoreElements()){
			String key = (String)e.nextElement();
			sessionbox.put(key, req.getParameterValues(key));
		}
		return sessionbox;
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static SessionBox getSessionBox(HttpServletRequest req)  {
		return getSessionBox(req, "shared session box");
	}

	/**
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param name box name for this SessionBox
	 */
	public static SessionBox getSessionBox(HttpServletRequest req, String name)  {
		
		javax.servlet.http.HttpSession  session = req.getSession(true);
		
		SessionBox sessionbox = null;
		
		//Object o = session.getValue(name);
		Object o = session.getAttribute(name); // by chkim(2002-04-26)
		if ( o != null ) {
			if ( o instanceof SessionBox ) {
				sessionbox = (SessionBox)o;
			}
			else {
				//session.removeValue(name);
				session.removeAttribute(name); // by chkim(2002-04-26)
			}
		}

		if ( sessionbox == null ) {
			sessionbox = new SessionBox(session, name);
			//session.putValue(name, sessionbox);
			session.setAttribute(name, sessionbox); // by chkim(2002-04-26)
		}
		
		Enumeration e = req.getParameterNames();
		while(e.hasMoreElements()){
			String key = (String)e.nextElement();
			sessionbox.put(key, req.getParameterValues(key));
		}
		return sessionbox;
	}

	/**
	 * 
	 * @return boolean
	 * @param req HttpServletRequest
	 */
	public static boolean isOverIE50(HttpServletRequest req) {
		String user_agent = (String) req.getHeader("user-agent");

		if ( user_agent == null ) 	return false;

		int index = user_agent.indexOf("MSIE");
		if ( index == -1 ) return false;

		int version = 0;
		try {
			version = Integer.parseInt(user_agent.substring(index+5, index+5+1));
		}
		catch(Exception e){}
		if ( version < 5 ) return false;
		
		return true;
	}
}