package kal.base.jdf.serv;

/**
 * 
 * @(#) .java
 * Copyright 1999-2000 by  LG-EDS Systems, Inc.,
 * Information Technology Group, Application Architecture Team, 
 * Application Intrastructure Part.
 * 236-1, Hyosung-2dong, Kyeyang-gu, Inchun, 407-042, KOREA.
 * All rights reserved.
 *  
 * NOTICE !      You can copy or redistribute this code freely, 
 * but you should not remove the information about the copyright notice 
 * and the author.
 *  
 * @author  WonYoung Lee, wyounglee@lgeds.lg.co.kr.
 */
import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import kal.base.com.util.StringUtil;
import kal.base.jdf.com.*;
import kal.base.jdf.com.Logger;
import kal.base.jdf.com.Message;
import kal.base.sys.dbbean.SysA010BG;

public abstract class JDFBaseServlet extends javax.servlet.http.HttpServlet {
	protected static String lineSeparator  = System.getProperty("line.separator");

	/**
	 * BaseServlet constructor comment.
	 */
	public JDFBaseServlet() {
		super();
	}

	/**
	 * Performs the HTTP GET operation; the default implementation
	 * reports an HTTP BAD_REQUEST error.  Overriding this method to
	 * support the GET operation also automatically supports the HEAD
	 * operation.  (HEAD is a GET that returns no body in the response;
	 * it just returns the request HEADer fields.)
	 *
	 * <p>Servlet writers who override this method should read any data
	 * from the request, set entity headers in the response, access the
	 * writer or output stream, and, finally, write any response data.
	 * The headers that are set should include content type, and
	 * encoding.  If a writer is to be used to write response data, the
	 * content type must be set before the writer is accessed.  In
	 * general, the servlet implementor must write the headers before
	 * the response data because the headers can be flushed at any time
	 * after the data starts to be written.
	 * 
	 * <p>Setting content length allows the servlet to take advantage
	 * of HTTP "connection keep alive".  If content length can not be
	 * set in advance, the performance penalties associated with not
	 * using keep alives will sometimes be avoided if the response
	 * entity fits in an internal buffer.
	 *
	 * <p>Entity data written for a HEAD request is ignored.  Servlet
	 * writers can, as a simple performance optimization, omit writing
	 * response data for HEAD methods.  If no response data is to be
	 * written, then the content length field must be set explicitly.
	 *
	 * <P>The GET operation is expected to be safe: without any side
	 * effects for which users might be held responsible.  For example,
	 * most form queries have no side effects.  Requests intended to
	 * change stored data should use some other HTTP method.  (There
	 * have been cases of significant security breaches reported
	 * because web-based applications used GET inappropriately.)
	 *
	 * <P> The GET operation is also expected to be idempotent: it can
	 * safely be repeated.  This is not quite the same as being safe,
	 * but in some common examples the requirements have the same
	 * result.  For example, repeating queries is both safe and
	 * idempotent (unless payment is required!), but buying something
	 * or modifying data is neither safe nor idempotent.
	 *
	 * @param req HttpServletRequest that encapsulates the request to
	 * the servlet 
	 * @param resp HttpServletResponse that encapsulates the response
	 * from the servlet
	 * 
	 * @exception IOException if detected when handling the request
	 * @exception ServletException if the request could not be handled
	 * 
	 * @see javax.servlet.ServletResponse#setContentType
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse res) 
		throws ServletException, IOException 
	{
		performBasePreTask(req, res);
	}

	/**
	 *
	 * Performs the HTTP POST operation; the default implementation
	 * reports an HTTP BAD_REQUEST error.  Servlet writers who override
	 * this method should read any data from the request (for example,
	 * form parameters), set entity headers in the response, access the
	 * writer or output stream and, finally, write any response data
	 * using the servlet output stream.  The headers that are set
	 * should include content type, and encoding.  If a writer is to be
	 * used to write response data, the content type must be set before
	 * the writer is accessed.  In general, the servlet implementor
	 * must write the headers before the response data because the
	 * headers can be flushed at any time after the data starts to be
	 * written.
	 *
	 * <p>If HTTP/1.1 chunked encoding is used (that is, if the
	 * transfer-encoding header is present), then the content-length
	 * header should not be set.  For HTTP/1.1 communications that do
	 * not use chunked encoding and HTTP 1.0 communications, setting
	 * content length allows the servlet to take advantage of HTTP
	 * "connection keep alive".  For just such communications, if
	 * content length can not be set, the performance penalties
	 * associated with not using keep alives will sometimes be avoided
	 * if the response entity fits in an internal buffer.
	 *
	 * <P> This method does not need to be either "safe" or
	 * "idempotent".  Operations requested through POST can have side
	 * effects for which the user can be held accountable.  Specific
	 * examples including updating stored data or buying things online.
	 *
	 * @param req HttpServletRequest that encapsulates the request to
	 * the servlet 
	 * @param resp HttpServletResponse that encapsulates the response
	 * from the servlet
	 * 
	 * @exception IOException if detected when handling the request
	 * @exception ServletException if the request could not be handled
	 *
	 * @see javax.servlet.ServletResponse#setContentType
	 */
	protected void doPost (HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		performBasePreTask(req, res);
	}

	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected void performBasePreTask (HttpServletRequest req, HttpServletResponse res) 
		throws ServletException, IOException {
		boolean trace = false;
		try {
			//Config conf = new Configuration();
			//trace = conf.getBoolean("kal.base.jdf.serv.baseservlet.trace");
			trace = true;
		}
		catch(Exception e) {}
		
		String returnPage = StringUtil.checkString(req.getParameter("returnPage"));

		String logMsg = null;
		long start = 0, end = 0;
		
		//ӽ÷ trace True  ==>  Server2 Restart Ʒ   2005.03.21
		//trace = true;
		
		if ( trace ) {
			logMsg = req.getRequestURI() + ":" + req.getRemoteHost() + "(" + req.getRemoteAddr() + ")";
			String user = req.getRemoteUser();
			if ( user != null ) logMsg += ":" + user;
			
			start = System.currentTimeMillis();
			Logger.sys.println(this, "[BASE2]"+logMsg + ":calling");
		}
		
		try {		
			HttpSession session = req.getSession();
			SysA010BG UserInfo = (SysA010BG)session.getAttribute("USERINFO");
			String empNo = "";
			if (UserInfo != null) empNo = StringUtil.checkString(UserInfo.getEmp_no());
			
			String flag = req.getParameter("flag");
			Logger.debug.println("************************************************************");
			Logger.debug.println("*  ID : "+empNo+"   Remote : "+req.getRemoteAddr()+"   Event : "+flag);
			Logger.debug.println("*  Class : "+req.getRequestURI());
			Logger.debug.println("************************************************************");
			
			// Task 
			if (flag != null) {
				if (flag.equalsIgnoreCase("select")) selectPreTask(req, res);
				else if (flag.equalsIgnoreCase("insert"))  insertPreTask(req, res);
			    else if (flag.equalsIgnoreCase("update")) updatePreTask(req,res); 
		        else if (flag.equalsIgnoreCase("delete")) deletePreTask(req, res);
				else if (flag.equalsIgnoreCase("check")) checkPreTask(req, res);
				else performPreTask(req, res);
			} else printMessagePage2(req,res," UNKNOWN SYSTEM Exception",returnPage,"check");
			//} else  performPreTask(req, res);
			//performPreTask(req, res);
		}
		catch(Exception e){
			java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
			java.io.PrintWriter writer = new java.io.PrintWriter(bos);
			e.printStackTrace(writer);
			writer.flush();
			String errString = "Programmer's Exception: " +logMsg +  lineSeparator + bos.toString();
			Logger.err.println(this, errString);
			printMessagePage2(req,res,errString,returnPage,"check");
		}

		if ( trace ) {
			end = System.currentTimeMillis();
			Logger.sys.println(this, logMsg + ":end(elapsed=" + (end-start) + ")" + lineSeparator);
		}
	}

	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void performPreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void selectPreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void updatePreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void deletePreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void insertPreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	/**
	 * 
	 * @param req HttpServletRequest
	 * @param res HttpServletResponse
	 */
	protected abstract void checkPreTask (HttpServletRequest req, HttpServletResponse res) throws Exception;

	/**
	* Sends a temporary redirect response to the client using the
	* specified redirect location URL.  The URL must be absolute (for
	* example, <code><em>https://hostname/path/file.html</em></code>).
	* Relative URLs are not permitted here.
	*
	* @param req javax.servlet.http.HttpServletRequest
	* @param res javax.servlet.http.HttpServletResponse
	* @param location the redirect location URL
	* @exception IOException If an I/O error has occurred.
	*/
	protected void printHtmlPage (HttpServletRequest req, HttpServletResponse res, String location) {
		try {
			res.sendRedirect(location);
		}
		catch (IOException e) {
			String err = e.getMessage();
			if ( err == null ) err = "";
			else err = " (" + err + ")";
			printMessagePage(req,res, "File Not Found : " + location + err );
		}
	}

	/**
	* Sends a temporary redirect response to the client using the
	* specified redirect location of jsp file.  The URL must be absolute (for
	* example, <code><em>/example/result.jsp</em></code>).
	* Relative URLs are not permitted here.
	*
	* @param req javax.servlet.http.HttpServletRequest
	* @param res javax.servlet.http.HttpServletResponse
	* @param jspfile the redirect location URL of jsp file.
	*/
	protected void printJspPage (HttpServletRequest req, HttpServletResponse res, String jspfile) {
		try {

			// IBM WebSphere 2.0.x
			//((com.sun.server.http.HttpServiceResponse) res).callPage(jspfile, req);

			// Servlet 2.1	
			
			//Logger.debug.println("[JDFBaseServlet] jspfile : "+jspfile);
			
			RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(jspfile);
			dispatcher.forward(req, res);
		}
		catch (IllegalStateException e) {
			try{
				java.io.PrintWriter out = res.getWriter();
				java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
				java.io.PrintWriter writer = new java.io.PrintWriter(bos);
				writer.println("JSP Call Error: " + this.getClass().getName() );
				writer.println("Request URI: " + req.getRequestURI() );
				String user = req.getRemoteUser();
				if ( user != null ) writer.println("User : " + user );
				writer.println("User Location  : " + req.getRemoteHost() + "(" + req.getRemoteAddr() + ")");
				e.printStackTrace(writer);
				writer.flush();
				res.setContentType("text/html;charset=euc-kr");
				out.println("<html><head><title>Errort</title></head><body bgcolor=white><xmp>");
				out.println(bos.toString());
				out.println("</xmp></body></html>");
				out.close();
			}
			catch(Exception ex){}
		}
		catch (IOException e) {
			try{
				java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
				java.io.PrintWriter writer = new java.io.PrintWriter(bos);
				writer.println("JSP Call Error: " + this.getClass().getName() );
				writer.println("Request URI: " + req.getRequestURI() );
				String user = req.getRemoteUser();
				if ( user != null ) writer.println("User : " + user );
				writer.println("User Location  : " + req.getRemoteHost() + 
				    "(" + req.getRemoteAddr() + ")");
				//e.printStackTrace(writer);
				writer.println(e.toString() + 
				  ":(client may have terminated connection before server completed response)" );
				writer.flush();
				Logger.debug.println(bos.toString());
			}
			catch(Exception ex){}
		}
		catch (Exception e) {
			java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();
			java.io.PrintWriter writer = new java.io.PrintWriter(bos);
			writer.println("JSP Call Error: " + this.getClass().getName() );
			writer.println("Request URI: " + req.getRequestURI() );
			String user = req.getRemoteUser();
			if ( user != null ) writer.println("User : " + user );
			writer.println("User Location  : " + req.getRemoteHost() + "(" + req.getRemoteAddr() + ")");
			e.printStackTrace(writer);
			writer.flush();
			Logger.sys.println(bos.toString());
			printMessagePage(req,res, bos.toString());
		}
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param msg java.lang.String
	 */
	protected void printMessagePage (HttpServletRequest req, HttpServletResponse res, Message msg) {
		printMessagePage(req,res,msg.toString(), (String)null);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param e Exception: ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printMessagePage (HttpServletRequest req, HttpServletResponse res, 
		Message user_msg, Exception e) 
	{
		printMessagePage(req, res, user_msg.toString(), kal.base.jdf.com.Utility.getStackTrace(e));
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param debug_msg ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printMessagePage (HttpServletRequest req, HttpServletResponse res, 
		Message user_msg, String debug_msg) 
	{
		printMessagePage(req, res, user_msg.toString(), debug_msg);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param msg ڿ  ޼
	 */
	protected void printMessagePage (HttpServletRequest req, HttpServletResponse res, String msg) 
	{
		printMessagePage(req,res, msg, (String)null);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param e Exception: ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printMessagePage (HttpServletRequest req, HttpServletResponse res, 
		String user_msg, Exception e) 
	{
		printMessagePage(req, res, user_msg, kal.base.jdf.com.Utility.getStackTrace(e));
	}

	/**
	 *  Method ݵ Ʈ ؼ ؾ . ֳĸ, Ʈ
	 * Message ִ ȭ ٸ  ֱ .
	 *
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param debug_msg ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected abstract void printMessagePage (HttpServletRequest req, HttpServletResponse res, String user_msg, String debug_msg);
	
	protected abstract void printMessagePage2(HttpServletRequest req, HttpServletResponse res, String user_msg, String next_page, String process_mode);

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param msg java.lang.String
	 */
	protected void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, Message msg) {
		printPopupMessagePage(req,res,msg.toString(), (String)null);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param e Exception: ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, 
		Message user_msg, Exception e) 
	{
		printPopupMessagePage(req, res, user_msg.toString(), kal.base.jdf.com.Utility.getStackTrace(e));
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param debug_msg ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, 
		Message user_msg, String debug_msg) 
	{
		printPopupMessagePage(req, res, user_msg.toString(), debug_msg);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param msg ڿ  ޼
	 */
	protected void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, String msg) 
	{
		printPopupMessagePage(req,res, msg, (String)null);
	}

	/**
	 * 
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param e Exception: ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, 
		String user_msg, Exception e) 
	{
		printPopupMessagePage(req, res, user_msg, kal.base.jdf.com.Utility.getStackTrace(e));
	}

	/**
	 *  Method ݵ Ʈ ؼ ؾ . ֳĸ, Ʈ
	 * Message ִ ȭ ٸ  ֱ .
	 *
	 * @param req javax.servlet.http.HttpServletRequest
	 * @param res javax.servlet.http.HttpServletResponse
	 * @param user_msg  ڿ  ޼
	 * @param debug_msg ߽ ڰ Debugging   ޼, 
	 *                                   ô  ʵ .
	 */
	protected abstract void printPopupMessagePage (HttpServletRequest req, HttpServletResponse res, String user_msg, String debug_msg);
}